package org.lym.pom.repository.mongo;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.mongodb.core.mapping.Field;

import java.io.Serializable;

/**
 * 朋友圈 / 状态 / 微博 / 说说这类，用于给用户记录分享当前时刻的类
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Document(collection = "moment")
public class Moment implements Serializable {

    /**
     * 评论 id
     */
    @Id
    private Object id;

    /**
     * 所属对象 id，可以是文章的
     * 评论的评论，该值仍然为被评论对象id
     */
    @Indexed
    private String relatedId;
    /**
     * 所属对象类型，如文章、动态、视频等，冗余信息用于扩展与提高读性能
     */
    @Field
    private String relatedType;
    /**
     * 所属对象的创建人id，冗余信息用于提高读性能
     */
    @Indexed
    private String relatedUserId;

    /**
     * 父节点id，默认0
     */
    @Indexed
    private String parentId;

    /**
     * 类型：评论、赞、喜欢
     */
    @Indexed
    private Integer type;

    /**
     * 内容
     */
    @Indexed
    private String content;

    /**
     * 评论人
     */
    @Indexed
    private String userId;


    /**
     * 创建时间
     */
    @Field
    private Long createTime;

    /**
     * 创建时间
     */
    @Field
    private Long updateTime;

}
