package org.lym.pom.service;

import org.lym.pom.repository.mongo.Follow;
import org.shoulder.core.dto.request.BasePageQuery;

/**
 * 评论/赞服务
 *
 * @author lym
 */
public interface FollowService {

    /**
     * 检查用户A是否关注了用户B
     *
     * @param userId    关注者的用户ID
     * @param followUserId 被关注者的用户ID
     */
    boolean isFollow(String userId, String followUserId);

    /**
     * 关注某个用户
     *
     * @param userId    用户ID
     * @param followUserId 被关注者的用户ID
     */
    void follow(String userId, String followUserId);

    /**
     * 取消关注某个用户
     *
     * @param userId    用户ID
     * @param followUserId 被关注者的用户ID
     */
    void unfollow(String userId, String followUserId);

    /**
     * 查询某个用户的关注列表
     *
     * @param userId    用户ID
     */
    BasePageQuery<Follow> queryFollowList(String userId);

    /**
     * 查询某个用户的被关注列表
     *
     * @param userId    用户ID
     */
    BasePageQuery<Follow> queryFollowedList(String userId);

    /**
     * 获取某个用户的关注目标数
     *
     * @param userId    用户ID
     * @return 关注数
     */
    long countFollowNumber(String userId);

    /**
     * 获取某个用户的被关注数
     *
     * @param userId    用户ID
     * @return 被关注数
     */
    long countFollowedNumber(String userId);


//    List<Follow> recommendFollower(String followUserId);
//    List<Follow> friendFollowers(String userId, String followUserId);
}
